## Imports
import re # built-in
import markdown, yaml #PIP
import extension # local

## Config
FORMAT_HTML = False # Useful for debugging, can be disabled otherwise.
CONFIG_REGEX = r"---((.|\n)+)---" # Regex used to find YAML config.
CATAGORY_REGEX = r"(?:^|\n)##\s([^\n]*)\n(.*?)(?=\n##?\s|$)" # Regex used to find each catagory - source: https://stackoverflow.com/a/66619938.

## Main
f = open("template.html")
MAIN_HTML = f.read()
f.close()

md = markdown.Markdown(
	extensions=[
		extension.ArticleMarkdown() # The extension which adds various features to the markdown
	]
)

def convertMarkdown(inputStr, fileName):
	## Variable Setup
	# Get Markdown config
	configReg = re.match(CONFIG_REGEX, inputStr, re.MULTILINE)
	if configReg is None:
		raise Exception("Config regex failed")
	config = yaml.safe_load(configReg.group(1))

	# Get Markdown catagories
	catagories = {}
	catagoriesReg = re.finditer(CATAGORY_REGEX, inputStr, re.DOTALL)
	if catagoriesReg is None:
		raise Exception("Catagories regex failed")
	for _, match in enumerate(catagoriesReg, start=1):
		catagories[match.group(1)] = match.group(2)

	## Document generation
	# Futher reading
	further = ""
	if "Further" in catagories:
		further = """<hr>
<div class="futher">
<h4>Further Reading:</h4>
	{further}
</div>
""".format(further = md.convert(catagories['Further']))

	# Sources
	sources = ""
	if "Sources" in catagories:
		a = ""
		for i in catagories['Sources'].split("\n"):
			if i != "":
				result = re.match(r"(\d+)\. (.+)", i)
				a += """<li id="source{number}">{content}</li>""".format(
					number = result.group(1),
					content = md.convert(result.group(2))
				)

		sources = f"""<hr>
<div class="sources">
	<h4>Sources:</h4>
	<ol>{a}
	</ol>
</div>"""
	
	# Footer
	footer = "<hr>"
	if "created" in config:
		footer += f"<b>This article was created on {config['created']}</b></br>"
	if "modified" in config:
		footer += f"<b>This article was last modified on {config['modified']}</b></br>"
	if "version" in config:
		footer += f"<b>This article is made for version {config['version']} of the software</b>"

	html = MAIN_HTML.format(
		title = config['title'],
		fileName = fileName,
		spywareLevel = config['rating'],
		introduction = md.convert(catagories['Introduction']),
		content = md.convert(catagories['Content']),
		further = further,
		sources = sources,
		footer = footer
	)

	return html

open("chrome.html", "w", encoding="utf8").write(
	convertMarkdown(
		open("chrome.md", encoding="utf8").read(), 
		"chrome"
	)
)